$title  GTAP Aggregation Routine -- Aggregates from any dimension dataset.

$ontext

-----------------------------------------------------------------------------------
Thomas F. Rutherford
Department of Economics

version July, 2003 (based on GDX format)
-----------------------------------------------------------------------------------
$offtext

*	Default archive format here:

$if set ds $set dataset %ds%
$if set dataset $goto defined
$log	Error: dataset not defined.  
$log	
$log	Syntax is: gams gtapaggr --dataset=ident
$abort

$label defined
$if exist ..\defines\%dataset%.map $goto exists
$log	Error: mapping file does not exist: ..\defines\%dataset%.map
$exit

$label exists
$call gams gtapsets --dataset=%dataset%

*       Read the source sets ii and rr (these were generated by gtapsets):

$include gtapaggr.set

*       Read the target sets:

$include ..\defines\%dataset%.set

*       Declare the mappings so that we do domain checking on input:

set     mapr(rr,r), mapi(ii,i);

*       Read the mappings:

$include ..\defines\%dataset%.map

SET     D Classes of demand for which import shares are differentiated / 
        G       Public, 
        C       Private Consumption,
        I       Intermediate input /;

ALIAS (I,J), (R,S), (D,DD), (JJ,II), (SS,RR);

*       Assign default mappings if they are not defined:

$if not defined MAPI MAPI(II,I)$SAMEAS(II,I)  = YES;
$if not defined MAPR MAPR(RR,R)$SAMEAS(RR,R)  = YES;

$if not declared FF   ALIAS (F,FF);
$if not declared MAPF SET MAPF(F,FF); MAPF(F,F)  = YES;

SET MAPJ(JJ,J); MAPJ(JJ,J) = MAPI(JJ,J);
SET MAPS(SS,S); MAPS(SS,S) = MAPR(SS,S);
SET MAPD(D,DD); MAPD(D,D)  = YES;

*       Read the source data:

parameter
        ty(ii,rr)       Output tax
        ti(jj,ii,rr)    Intermediate input tax
        tf(ff,ii,rr)    Factor tax
        tx(ii,ss,rr)    Export tax rate (defined on a net basis)
        tm(ii,ss,rr)    Import tariff rate
        tg(ii,rr)       Tax rates on government demand
        tp(ii,rr)       Tax rate on private demand

	evf(ii,jj,rr)	Volume of energy inputs by firms (MTOE),
	evh(ii,rr)	Volume of energy inputs by households (MTOE),
	evt(ii,rr,ss)	Volume of energy trade (MTOE),

        vafm(jj,ii,rr)  Aggregate intermediate inputs
        vfm(ff,ii,rr)   Value of factor inputs (net of tax)
        vxmd(ii,rr,ss)  Value of commodity trade (fob - net export tax)
        vtwr(ii,rr,ss)  Transport services
        vst(ii,rr)      Value of international transport sales
        vdgm(ii,rr)     Government demand (domestic)
        vigm(ii,rr)     Government demand (imported)
        vdpm(ii,rr)     Aggregate private demands (domestic)
        vipm(ii,rr)     Aggregate private demands (domestic)

	vsave(rr)	Regional savings
	vkb(rr)		Value of capital stock (beginning of period)
	vdep(rr)	Value of capital depreciation

        ty0(i,r)        Output tax
        ti0(j,i,r)      Intermediate input tax
        tf0(f,i,r)      Factor tax
        ti0(i,j,r)      Intermediate input tax
        tx0(i,s,r)      Export tax rate (defined on a net basis)
        tm0(i,s,r)      Import tariff rate
        tg0(i,r)        Tax rates on government demand
        tp0(i,r)        Tax rate on private demand

        vafm0(j,i,r)    Aggregate intermediate inputs
        vfm0(f,i,r)     Value of factor inputs (net of tax)
        vxmd0(i,r,s)    Value of commodity trade (fob - net export tax)
        vtwr0(i,r,s)    Transport services
        vst0(i,r)       Value of international transport sales
        vdgm0(i,r)      Government demand (domestic)
        vigm0(i,r)      Government demand (imported)
        vdpm0(i,r)      Aggregate private demands (domestic)
        vipm0(i,r)      Aggregate private demands (domestic)

	vsave0(r)	Regional savings
	vkb0(r)		Value of capital stock (beginning of period)
	vdep0(r)	Value of capital depreciation,

	evf0(i,j,r)	Volume of energy inputs by firms (MTOE),
	evh0(i,r)	Volume of energy inputs by households (MTOE),
	evt0(i,r,s)	Volume of energy trade (MTOE);



set	r78(*)		Set of regions in the original database

$log Reading GTAP Data...

$gdxin ..\data\%source%.gdx
$load   ty ti tf tx tm tg tp vafm vfm vxmd vtwr vst vdgm vigm vdpm vipm vsave vdep vkb evf evh evt r78 

set	map78(r78,rr)	Mapping from original GTAP regions to the source regions ;
$load   map78

*       Aggregate the value arrays:

$log Aggregating GTAP Data...

$batinclude aggr vafm j i r vafm0
$batinclude aggr vfm  f i r vfm0
$batinclude aggr vxmd i r s vxmd0
$batinclude aggr vtwr i r s vtwr0
$batinclude aggr vst  i r   vst0
$batinclude aggr vdgm i r   vdgm0
$batinclude aggr vigm i r   vigm0
$batinclude aggr vdpm i r   vdpm0
$batinclude aggr vipm i r   vipm0
$batinclude aggr vsave r   vsave0
$batinclude aggr vkb   r   vkb0
$batinclude aggr vdep  r   vdep0

$batinclude aggr evf j i r evf0
$batinclude aggr evh i r evh0
$batinclude aggr evt i r s evt0

*       Convert tax rates into values for aggregation:

parameter
        vxm(ii,rr)      Value of exports
        vdm(ii,rr)      Value of domestic production
        etax(ff,jj,ss)  Value of taxes on factor inputs to production
        mtax(ii,rr,ss)  Value of import tariffs
        xtax(ii,rr,ss)  Value of export taxes
        ytax(ii,rr)     Value of taxes on production outputs
        vtg(ii,rr)      Tax on government demand
        vti(jj,ii,rr)   Intermediate input tax
        vtp(ii,rr)      Taxes on private demand

        vxm0(i,r)       Value of exports
        vdm0(i,r)       Value of domestic production
        vgm0(i,r)       Value of public consumption
        vpm0(i,r)       Value of private consumption
        etax0(f,j,s)    Value of taxes on factor inputs to production
        mtax0(i,r,s)    Value of import tariffs
        xtax0(i,r,s)    Value of export taxes
        ytax0(i,r)      Value of taxes on production outputs
        vtg0(i,r)       Tax on government demand
        vti0(j,i,r)     Intermediate input tax
        vtp0(i,r)       Taxes on private demand;


vxm(ii,rr) = sum(ss, vxmd(ii,rr,ss));
vdm(ii,rr) = max(0, (sum(jj, vafm(jj,ii,rr)*(1+ti(jj,ii,rr))) 
                   + sum(ff, vfm(ff,ii,rr)*(1+tf(ff,ii,rr))))/(1-ty(ii,rr)) - vxm(ii,rr));

xtax(ii,rr,ss)$tx(ii,rr,ss) = tx(ii,rr,ss)*vxmd(ii,rr,ss);
mtax(ii,ss,rr)$tm(ii,ss,rr) = tm(ii,ss,rr)*(vxmd(ii,ss,rr)+xtax(ii,ss,rr)+vtwr(ii,ss,rr));
etax(ff,ii,rr)$tf(ff,ii,rr) = tf(ff,ii,rr)*vfm(ff,ii,rr);
vtp(ii,rr)$tp(ii,rr) = tp(ii,rr)*(vipm(ii,rr)+vdpm(ii,rr));
vtg(ii,rr)$tg(ii,rr) = tg(ii,rr)*(vigm(ii,rr)+vdgm(ii,rr));
vti(jj,ii,rr)$ti(jj,ii,rr) = ti(jj,ii,rr)*vafm(jj,ii,rr);
ytax(ii,rr)$ty(ii,rr) = ty(ii,rr) * (vxm(ii,rr)+vdm(ii,rr));

$batinclude aggr xtax i r s  xtax0
$batinclude aggr mtax i s r  mtax0
$batinclude aggr etax f i r  etax0
$batinclude aggr vtp  i r    vtp0
$batinclude aggr vdm  i r    vdm0
$batinclude aggr vtg  i r    vtg0
$batinclude aggr vti  j i r  vti0
$batinclude aggr ytax i r    ytax0

*       Reconvert to rates:

vxm0(i,r) = sum(s, vxmd0(i,r,s));
vgm0(i,r) = vigm0(i,r) + vdgm0(i,r);
vpm0(i,r) = vipm0(i,r) + vdpm0(i,r);

ty0(i,r)$(vdm0(i,r)+vxm0(i,r)) = ytax0(i,r) / (vdm0(i,r)+vxm0(i,r));
tx0(i,r,s)$vxmd0(i,r,s) = xtax0(i,r,s) / vxmd0(i,r,s);
tm0(i,s,r)$(vxmd0(i,s,r)+xtax0(i,s,r)+vtwr0(i,s,r))
        = mtax0(i,s,r)/(vxmd0(i,s,r)+xtax0(i,s,r)+vtwr0(i,s,r));
tf0(f,i,r)$vfm0(f,i,r) = etax0(f,i,r) / vfm0(f,i,r);
tg0(i,r)$vgm0(i,r)     = vtg0(i,r) / vgm0(i,r);
tp0(i,r)$vpm0(i,r)     = vtp0(i,r) / vpm0(i,r);
ti0(i,j,r)$vafm0(i,j,r) = vti0(i,j,r) / vafm0(i,j,r);

set map780(r78,r)	Mapping from the original GTAP regions to those in the aggregate dataset;
loop(map78(r78,rr),
	map780(r78,r)$mapr(rr,r) = yes;
);

*------------------------------------------------------------------------------
*       If the source data is purely economic statistics -- store arrays 
*	which have already been constructed and then complete the final report:

*       Move the dataset into a GDX archive:

execute_unload '..\data\%dataset%', ty0=ty, ti0=ti, tf0=tf, tx0=tx, tm0=tm, tg0=tg, tp0=tp, 
	vafm0=vafm, vfm0=vfm, vxmd0=vxmd, 
	vtwr0=vtwr, vst0=vst, vdgm0=vdgm, vigm0=vigm, vdpm0=vdpm, vipm0=vipm, vsave0=vsave, vdep0=vdep, 
	vkb0=vkb, i, r, f, r78, map780=map78,
	evf0=evf, evh0=evh, evt0=evt;
